bad_keywords_list=[
'AMPLIST',
'AMPNAME',
'ASCLDMED',
'ASCLDMEN',
'ASCLDMOD',
'ASCLDSTD',
'ASECA',
'ASECB',
'ASNERMED',
'ASNERMEN',
'ASNERMOD',
'ASNERSTD',
'ASNESMED',
'ASNESMEN',
'ASNESMOD',
'ASNESSTD',
'ASNWRMED',
'ASNWRMEN',
'ASNWRMOD',
'ASNWRSTD',
'ASNWSMED',
'ASNWSMEN',
'ASNWSMOD',
'ASNWSSTD',
'ASSERMED',
'ASSERMEN',
'ASSERMOD',
'ASSERSTD',
'ASSESMED',
'ASSESMEN',
'ASSESMOD',
'ASSESSTD',
'ASSPRMED',
'ASSPRMEN',
'ASSPRMOD',
'ASSPRSTD',
'ASSPSMED',
'ASSPSMEN',
'ASSPSMOD',
'ASSPSSTD',
'ASSWRMED',
'ASSWRMEN',
'ASSWRMOD',
'ASSWRSTD',
'ASSWSMED',
'ASSWSMEN',
'ASSWSMOD',
'ASSWSSTD',
'BIAS0',
'BIAS1',
'BIASSEC',
'BITPIX',
'BONANGLE',
'BSECA',
'BSECB',
'CALFOCUS',
'CCD',
'CCDBIN1',
'CCDBIN2',
'CCDNICK',
'CCDSUM',
'CMMTOBS',
'CMMTSEQ',
'COMMENT',
'CONSWV',
'CSECA',
'CSECB',
'DARKCUR',
'DARKTIME',
'DATE',
'DDSEEMED',
'DDSEEMEN',
'DDSEEMOD',
'DDSEESTD',
'DECOFF',
'DEFOCUS',
'DETECTOR',
'DETSECA',
'DETSECB',
'DETSTAT',
'DETTEM',
'DOMEAZ',
'DOMELOUV',
'DOMESHUT',
'DSECA',
'DSECB',
'EXPMEAS',
'EXPREQ',
'EXPTYPE',
'EXTVER',
'FILTERID',
'FOCUSID',
'FOCUSPOS',
'FSAGAIN',
'FSAMAXZ',
'FSAMEANZ',
'FSAMINZ',
'FSANMOVE',
'FSARATE',
'FSASTATE',
'FSASTDVZ',
'FSATHRES',
'GAIN',
'GAINA',
'GAINB',
'GCOUNT',
'GUIDEC',
'GUIDEC1',
'GUIDEC2',
'GUIDECPM',
'GUIDEPM1',
'GUIDEPM2',
'GUIEQUI1',
'GUIEQUI2',
'GUIEQUIN',
'GUIFLUX',
'GUIFLUX1',
'GUIFLUX2',
'GUIFWHX',
'GUIFWHX1',
'GUIFWHX2',
'GUIFWHY',
'GUIFWHY1',
'GUIFWHY2',
'GUIMAGN',
'GUIMAGN1',
'GUIMAGN2',
'GUINAME',
'GUINAME1',
'GUINAME2',
'GUIOBJN',
'GUIOBJN1',
'GUIOBJN2',
'GUIPOSX1',
'GUIPOSX2',
'GUIPOSY1',
'GUIPOSY2',
'GUIPOSZ1',
'GUIPOSZ2',
'GUIRA',
'GUIRA1',
'GUIRA2',
'GUIRADE1',
'GUIRADE2',
'GUIRADEC',
'GUIRAPM',
'GUIRAPM1',
'GUIRAPM2',
'HA',
'HAEND',
'HISTORY',
'HSTTIME',
'IMAGESWV',
'INHERIT',
'INSTMODE',
'INSTRUME',
'INSTZDEC',
'INSTZRA',
'ISUGAIN',
'ISURATE',
'ISUSTATE',
'ISUSTDVX',
'ISUSTDVY',
'ISUTCSOR',
'LATITUDE',
'LONGITUD',
'MAXLIN',
'MAXLINA',
'MAXLINB',
'MIRRCOV',
'MOONALT',
'MOONANGL',
'MOONAZ',
'MOONPHAS',
'MOONUP',
'NAXIS',
'NAXIS1',
'NAXIS2',
'NGUIDER',
'NGUISTAR',
'OBJDEC',
'OBJDECPM',
'OBJEQUIN',
'OBJMAG',
'OBJRA',
'OBJRADEC',
'OBJRAPM',
'OBSERVER',
'OBSID',
'OBSTYPE',
'ORIGIN',
'PATHNAME',
'PCOUNT',
'PIXSCAL1',
'PIXSCAL2',
'PIXSIZE1',
'PIXSIZE2',
'PI_NAME',
'PRESSURE',
'QCOORD',
'QEPOINTS',
'QICLABEL',
'QICSEQID',
'QID',
'QOBITER',
'QOBLABEL',
'QOBSEQID',
'QOBSERVE',
'QOFFID',
'QOGLABEL',
'RAOFF',
'RASTER',
'RDNOISE',
'RDNOISEA',
'RDNOISEB',
'REGMODE',
'RELHUMID',
'REL_DATE',
'ROOTEND',
'ROTANGLE',
'SATURATE',
'SHUTCLOS',
'SHUTOPEN',
'SKYFLUX',
'SPBZPMED',
'SPBZPMEN',
'SPBZPMOD',
'SPBZPSTD',
'SPOZPMED',
'SPOZPMEN',
'SPOZPMOD',
'SPOZPSTD',
'SPVZPMED',
'SPVZPMEN',
'SPVZPMOD',
'SPVZPSTD',
'TCSAMODE',
'TCSAPDEC',
'TCSAPHA',
'TCSEDDEC',
'TCSEDHA',
'TCSENDEC',
'TCSENHA',
'TCSEPICS',
'TCSGEDEC',
'TCSGEHA',
'TCSGPSBC',
'TCSGPSTM',
'TCSINDEC',
'TCSINHA',
'TCSLST',
'TCSMJD',
'TCSMVDEC',
'TCSMVRA',
'TCSMVX',
'TCSMVY',
'TCSOBDEC',
'TCSOBHA',
'TCSRBUSS',
'TCSSEDEC',
'TCSSEHA',
'TEA2INCH',
'TEA2INEB',
'TEA6FOOT',
'TEALOWWS',
'TEAMIRCI',
'TEAMIRCO',
'TEANRLSB',
'TEAPMCLW',
'TEAPMSPN',
'TEAPMSPS',
'TEATOPOP',
'TEATOPWS',
'TEATRNGE',
'TEATRNGW',
'TEAWTHRT',
'TELALT',
'TELAZ',
'TELCONF',
'TELESCOP',
'TELFOCUS',
'TELSTAT',
'TEMPERAT',
'TESCONRM',
'TESHRSET',
'TESPIERN',
'TESPIERS',
'TESPMIRE',
'TESPMIRS',
'TESPMIRW',
'TESTSPHE',
'TESTTREM',
'TESTTRNM',
'TESTTRSE',
'TESTTRSH',
'TESTTRSL',
'TESTTRWM',
'TIMESYS',
'TSECA',
'TSECB',
'VENTL1',
'VENTL2',
'VENTL3',
'VENTL4',
'VENTL5',
'VENTL6',
'VENTMODE',
'VENTR1',
'VENTR2',
'VENTR3',
'VENTR4',
'VENTR5',
'VENTR6',
'WDSEEMED',
'WDSEEMEN',
'WDSEEMOD',
'WDSEESTD',
'WINDDIR',
'WINDSCR',
'WINDSPED',
'WMSEEMED',
'WMSEEMEN',
'WMSEEMOD',
'WMSEESTD',
'XPROBE',
'XTENSION',
'YPROBE',
'ZPROBE',
],



import json
import os
__author__ = "David Rusk <drusk@uvic.ca>"

DEFAULT_CONFIG_FILE = "config.json"

_configs = {}


def read(keypath, configfile=None):
    """
    Reads a value from the configuration file.

    Args:
      keypath: str
        Specifies the key for which the value is desired.  It can be a
        hierarchical path.  Example: "section1.subsection.key1"
      configfile: str
        Path to the config file to read.  Defaults to None, in which case
        the application's default config file is used.

    Returns:
      value from configuration file
    """
    if configfile in _configs:
        appconfig = _configs[configfile]
    else:
        appconfig = AppConfig(configfile=configfile)
        _configs[configfile] = appconfig

    return appconfig.read(keypath)


class AppConfig(object):
    """
    Provides programmatic access to contents of the application
    configuration file.
    """

    def __init__(self, configfile=None):
        if configfile is None or not os.access(configfile, os.R_OK):
            configfile = self._get_default_config_file_path(configfile=configfile)

        with open(configfile, "rb") as filehandle:
            self._data = json.loads(filehandle.read())

    def _get_default_config_file_path(self, configfile=None):
        if configfile is None:
            configfile = DEFAULT_CONFIG_FILE
        return os.path.join(os.path.dirname(__file__), configfile)

    def read(self, keypath):
        curr_data = self._data
        for key in keypath.split("."):
            curr_data = curr_data[key]
        # Possibly overide the value with an OS variable.
        # print(f'{keypath} - '+str(os.getenv(f'{keypath}', None)))
        return os.getenv(f'MOP_{keypath.replace(".","_")}', curr_data)

    def unravel(self, d):
        list_of_keys = []
        for key in d:
            if isinstance(d[key], dict):
                for sub_key in self.unravel(d[key]):
                    list_of_keys.append(f'{key}.{sub_key}')
            else:
                list_of_keys.append(f'{key}')
        return list_of_keys

    def __str__(self):
        env_vars = self.unravel(self._data)
        _result = ""
        for key in env_vars:
            _result += f'MOP_{key.replace(".", "_")}: {self.read(key)}\n'
        return _result



import logging
# noinspection PyUnresolvedReferences
from mp_ephem import time_mpc
from astropy.time import Time
from astropy.table import Table
from ossos import storage


class MOPFile(object):
    """Read in a MOP formatted file"""

    def __init__(self, basename=None, ccd=None, extension=None, type='p', prefix=None, filename=None, subfmt='jmp'):
        """does nothing"""

        self.basename = basename
        self.ccd = ccd
        self.extension = extension
        self.type = type
        self.prefix = prefix
        self._filename = filename
        self.subfmt = subfmt

        self.header = None
        self.data = None
        self._parse()

    @property
    def filename(self):
        """
        Name if the MOP formatted file to parse.
        @rtype: basestring
        @return: filename
        """
        if self._filename is None:
            self._filename = storage.get_file(self.basename,
                                              self.ccd,
                                              ext=self.extension,
                                              version=self.type,
                                              prefix=self.prefix)
        return self._filename

    @property
    def usecols(self):
        """
        The column indexies to read from the file.

        This is only needed for the 'matt' format files as they have a long standing format error.

        @rtype list
        @return: list of column indexies
        """
        if 'matt' in self.filename:
            return list(range(5))
        return None

    def _parse(self):
        """read in a file and return a MOPFile object."""
        with open(self.filename, 'r') as fobj:
            lines = fobj.read().split('\n')

        # Create a header object with content at start of file
        self.header = MOPHeader(self.subfmt).parser(lines)

        # Create a data attribute to hold an astropy.table.Table object
        self.data = MOPDataParser(self.header).parse(lines)


class MOPDataParser(object):
    """ A MOPFile Data object"""

    def __init__(self, header):
        """

        @param header: MOPHeader
        """
        self.header = header
        self._table = None

    @property
    def table(self):
        """
        The astropy.table.Table object that will contain the data result
        @rtype: Table
        @return: data table
        """
        if self._table is None:
            column_names = []
            for fileid in self.header.file_ids:
                for column_name in self.header.column_names:
                    column_names.append("{}_{}".format(column_name, fileid))
                column_names.append("ZP_{}".format(fileid))
            if len(column_names) > 0:
                self._table = Table(names=column_names)
            else:
                self._table = Table()
        return self._table

    def parse(self, lines):

        record = []
        while len(lines) > 0:
            line = lines.pop(0)
            if line.strip().startswith("#"):
                continue
            if len(line.strip()) != 0:
                record.append(line)
                continue
            if len(record) == 0:
                continue
            if len(record) != len(self.header.file_ids):
                logging.debug("record: {}".format(record))
                logging.debug("file_ids: {}".format(self.header.file_ids))
                raise ValueError("Wrong number of entries in record.")
            values = []
            record_number = 0
            for line in record:
                if len(line.split()) != len(self.header.column_names):
                    logging.debug("line: {}".format(line))
                    logging.debug("Wrong number of columns compared to: {}".format(self.header.column_names))
                    raise ValueError("column length mismatch")
                values.extend([ float(x) for x in line.split() ])
                try:
                    zp = float(open("{}.zeropoint.used".format(self.header.file_ids[record_number]), 'r').readline())
                except:
                    zp = 0
                values.append(zp)
                record_number += 1
            self.table.add_row(values)
            record = []
        return self.table


class MOPHeader(object):
    """A MOPFile Header object"""

    def __init__(self, subfmt):

        self.keywords = {}
        self.subfmt = subfmt
        self.file_ids = []
        self.column_names = []
        self._file_id_idx = 0

    def __str__(self):
        return str(self.keywords)+'\n'+str(self.column_names)

    def parser(self, lines):
        """Given a set of lines parse the into a MOP Header"""
        while len(lines) > 0:
            if lines[0].startswith('##') and lines[1].startswith('# '):
                # A two-line keyword/value line starts here.
                self._header_append(lines.pop(0), lines.pop(0))
            elif lines[0].startswith('# '):
                # Lines with single comments are exposure numbers unless preceeded by double comment line
                self._append_file_id(lines.pop(0))
            elif lines[0].startswith('##'):
                # Double comment lines without a single comment following are column headers for dataset.
                self._set_column_names(lines.pop(0)[2:])
            else:
                # Last line of the header reached.
                return self
        raise IOError("Failed trying to read header")

    def _compute_mjd(self, kw, val):
        """
        Sometimes that MJD-OBS-CENTER keyword maps to a three component string, instead of a single value.
        @param kw: a list of keywords
        @param val: a list of matching values
        @return: the MJD-OBS-CENTER as a julian date.
        """
        try:
            idx = kw.index('MJD-OBS-CENTER')
        except ValueError:
            return
        if len(val) == len(kw):
            return
        if len(val) - 2 != len(kw):
            raise ValueError("convert: keyword/value lengths don't match: {}/{}".format(kw, val))
        val.insert(idx, Time("{} {} {}".format(val.pop(idx), val.pop(idx), val.pop(idx)),
                             format='mpc',
                             scale='utc').mjd)
        logging.debug("Computed MJD: {}".format(val[idx]))

    def _header_append(self, kw_line, val_line):
        
        keywords = kw_line.strip().split()
        values = val_line.strip().split()
        self._compute_mjd(keywords, values)
        self._append_keywords(keywords, values)
        return

    def _append_keywords(self, keywords, values):

        if len(keywords) != len(values):
            raise ValueError("keyword/value lengths don't match: {}/{}".format(keywords, values))
        file_id = self.file_ids[self._file_id_idx]
        while len(keywords) > 0:
            keyword = keywords.pop()
            value = values.pop()
            if keyword not in self.keywords :
                self.keywords[keyword] = []
            self.keywords[keyword].append(value)
        return

    def _append_file_id(self, line):
        self.file_ids.append(line.split()[1].strip())

    def _set_column_names(self, line):
        if self.subfmt=='matt':
            self.column_names = line[1:].split()[0:5]
        else:
            self.column_names = line[1:].split()



url="""http://www.cadc-ccda.hia-iha.nrc-cnrc.gc.ca/tap/sync"""
params = {"LANG": "ADQL",
          "REQUEST": "doQuery",
          "FORMAT": "csv" }

QUERY="""SELECT Observation.sequenceNumber AS expnum FROM caom2.Plane AS Plane JOIN caom2.Observation AS Observation ON Plane.obsID = Observation.obsID WHERE  ( Observation.instrument_name = 'MegaPrime' AND Observation.collection = 'CFHT' AND INTERSECTS( INTERVAL( 57754.0, 1.7976931348623157E308 ), Plane.time_bounds ) = 1 AND lower(Observation.proposal_id) IN ('17bt01', '17bs02', '17ad96', '17bf14', '17bc09' ) AND  Plane.calibrationLevel=1 AND ( Plane.quality_flag IS NULL OR Plane.quality_flag != 'junk') ) ORDER BY Observation.sequenceNumber """

params["QUERY"] = QUERY

import argparse
import smtplib
from email.mime.text import MIMEText
import requests
from io import StringIO
from astropy.table import Table
from astropy.io import ascii
import tempfile
import vos

parser = argparse.ArgumentParser()
parser.add_argument('-o','--output', action="store_true", help="Store to file instead of mailing list")

args = parser.parse_args()

current_list = vos.Client().listdir('vos:OSSOS/dbimages')
recon_exposures = Table.read(StringIO(requests.get(url, params=params).content), format='ascii.csv')
print(recon_exposures)

if args.output:
    with open('list.txt', 'w') as fout:
        cond = [ str(g) not in current_list for g in recon_exposures['expnum']]
        ascii.write(recon_exposures[cond], fout, format='no_header')

if not args.output:
    you = 'Stephen.Gwyn@nrc-cnrc.gc.ca'
    you = 'jj.kavelaars@nrc-cnrc.gc.ca'
    me = 'jjk@uvic.ca'

    with tempfile.NamedTemporaryFile() as fout:
        fout.write("Hi Stephen, \n Here is correct list of OSSOS exposures that need processing (PITCAIRN+ASTGWYN).\n\nThanks\nJJ\n")
        cond = [ str(g) not in current_list for g in recon_exposures['expnum']]
        ascii.write(recon_exposures[cond], fout, format='no_header', overwrite=True)
        fout.seek(0)
        msg = MIMEText(fout.read())
        msg['Subject'] = "OSSOS Exposures to process"
        msg['From'] = me
        msg['To'] = you
        msg['CC'] = me
    

    s = smtplib.SMTP('smtp.uvic.ca')
    s.sendmail(me, [you, me], msg.as_string())
    s.quit()





###############################################################################
#                                                                            ##
# Copyright 2013 by its authors                                              ##
# See COPYING, AUTHORS                                                       ##
#                                                                            ##
# This file is part of OSSOS Moving Object Pipeline (OSSOS-MOP)              ##
#                                                                            ##
#    OSSOS-MOP is free software: you can redistribute it and/or modify       ##
#    it under the terms of the GNU General Public License as published by    ##
#    the Free Software Foundation, either version 3 of the License, or       ##
#    (at your option) any later version.                                     ##
#                                                                            ##
#    OSSOS-MOP is distributed in the hope that it will be useful,            ##
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          ##
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           ##
#    GNU General Public License for more details.                            ##
#                                                                            ##
#    You should have received a copy of the GNU General Public License       ##
#    along with OSSOS-MOP.  If not, see <http://www.gnu.org/licenses/>.      ##
#                                                                            ##
###############################################################################
"""Run step2 of the OSSOS pipeline."""

import argparse
import errno
import logging
import math
import os
from astropy.io import fits
from ossos import storage
from ossos import util
from ossos import wcs
import sys

_FWHM = 4.0

task = util.task()
dependency = "step1"


def compute_trans(expnums, ccd, version, prefix=None, default="WCS"):
    """
    Pull the astrometric header for each image, compute an x/y transform and compare to trans.jmp

    this one overides trans.jmp if they are very different.
    @param expnums: three exposures to link together.
    @param ccd: which CCD of the mosaic to process
    @param version: which version of the exposure: o == raw, p == processed, s == scrambled
    @param prefix: string prefixed to the filenames (fk for planted images, blank otherwise)
    @param default: which transform to use ? WCS or JPM?
    @return: None
    """
    wcs_dict = {}
    for expnum in expnums:
        try:
            # TODO This assumes that the image is already N/E flipped.
            # If compute_trans is called after the image is retrieved from archive then we get the disk version.
            filename = storage.get_image(expnum, ccd, version, prefix=prefix)
            this_wcs = wcs.WCS(fits.open(filename)[0].header)
        except Exception as err:
            logging.warning("WCS Trans compute failed. {}".format(str(err)))
            return
        wcs_dict[expnum] = this_wcs
    x0 = wcs_dict[expnums[0]].header['NAXIS1'] / 2.0
    y0 = wcs_dict[expnums[0]].header['NAXIS2'] / 2.0
    (ra0, dec0) = wcs_dict[expnums[0]].xy2sky(x0, y0)
    result = ""
    for expnum in expnums:
        (x, y) = wcs_dict[expnum].sky2xy(ra0, dec0)
        dr = 1000
        try:
            filename = storage.get_file(expnum, ccd, version, ext='.trans.jmp', prefix=prefix)
            jmp_trans = open(filename, 'r').readline().split()
            x1 = float(jmp_trans[0]) + float(jmp_trans[1]) * x + float(jmp_trans[2]) * y
            y1 = float(jmp_trans[3]) + float(jmp_trans[4]) * x + float(jmp_trans[5]) * y
            dr = math.sqrt((x1 - x0) ** 2 + (y1 - y0) ** 2)
        except Exception:
            pass
        if dr > 0.5:
            result += "WARNING: WCS-JMP transforms mis-matched {} reverting to using {}.\n".format(expnum, default)
            if default == "WCS":
                uri = storage.dbimages_uri(expnum, ccd, version, ext='.trans.jmp', prefix=prefix)
                filename = os.path.basename(uri)
                trans = open(filename, 'w')
                trans.write("{:5.2f} 1. 0. {:5.2f} 0. 1.\n".format(x0 - x, y0 - y))
                trans.close()
        else:
            result += "WCS-JMP transforms match {}\n".format(expnum)
    return result


def run(expnums, ccd, version, prefix=None, dry_run=False, default="WCS", force=False):
    """run the actual step2  on the given exp/ccd combo"""

    jmp_trans = ['step2ajmp']
    jmp_args = ['step2bjmp']
    matt_args = ['step2matt_jmp']

    if storage.get_status(task, prefix, expnums[0], version, ccd) and not force:
        logging.info("{} completed successfully for {}{}{}{:02d}".format(
            task, prefix, expnums[0], version, ccd))
        return

    with storage.LoggingManager(task, prefix, expnums[0], ccd, version, dry_run):
        try:
            for expnum in expnums:
                if not storage.get_status(dependency, prefix, expnum, version=version, ccd=ccd):
                    raise IOError(35, "Cannot start {} as {} not yet completed for {}{}{}{:02d}".format(
                        task, dependency, prefix, expnum, version, ccd))
            message = storage.SUCCESS

            idx = 0
            logging.info("Retrieving catalog files to do matching.")
            for expnum in expnums:
                jmp_args.append(
                    storage.get_file(expnum, ccd=ccd, version=version, ext='obj.jmp', prefix=prefix)[0:-8]
                )
                jmp_trans.append(
                    storage.get_file(expnum, ccd=ccd, version=version, ext='obj.jmp', prefix=prefix)[0:-8]
                )
                idx += 1
                matt_args.append('-f%d' % idx)
                matt_args.append(
                    storage.get_file(expnum, ccd=ccd, version=version, ext='obj.matt', prefix=prefix)[0:-9]
                )

            logging.info("Computing the catalog alignment using sources in catalogs.")
            try:
                logging.info(util.exec_prog(jmp_trans))
                if default == "WCS":
                    logging.info("Comparing computed transform to WCS values")
                    logging.info(compute_trans(expnums, ccd, version, prefix, default=default))
            except Exception as ex:
                logging.info("JMP Trans failed: {}".format(ex))
                logging.info(compute_trans(expnums, ccd, version, prefix, default="WCS"))

            logging.info("Using transform to match catalogs for three images.")
            logging.info(util.exec_prog(jmp_args))
            logging.info(util.exec_prog(matt_args))

            # check that the shifts from step2 are rational by matching the bright star lists.
            logging.info("Uisng checktrans to ensure that transforms were reasonable.")
            check_args = ['checktrans']
            if os.access('proc-these-files', os.R_OK):
                os.unlink('proc-these-files')
            ptf = open('proc-these-files', 'w')
            ptf.write("# A dummy file that is created so checktrans could run.\n")
            ptf.write("# Frame FWHM PSF?\n")
            for expnum in expnums:
                filename = os.path.splitext(storage.get_image(expnum, ccd, version=version, prefix=prefix))[0]
                if not os.access(filename + ".bright.psf", os.R_OK):
                    os.link(filename + ".bright.jmp", filename + ".bright.psf")
                if not os.access(filename + ".obj.psf", os.R_OK):
                    os.link(filename + ".obj.jmp", filename + ".obj.psf")
                ptf.write("{:>19s}{:>10.1f}{:>5s}\n".format(filename,
                                                            _FWHM,
                                                            "NO"))
            ptf.close()
            if os.access('BAD_TRANS', os.F_OK):
                os.unlink('BAD_TRANS')

            logging.info(util.exec_prog(check_args))

            if os.access('BAD_TRANS', os.F_OK):
                raise OSError(errno.EBADMSG, 'BAD_TRANS')

            if os.access('proc-these-files', os.F_OK):
                os.unlink('proc-these-files')

            if dry_run:
                return

            for expnum in expnums:
                for ext in ['unid.jmp', 'unid.matt', 'trans.jmp']:
                    uri = storage.dbimages_uri(expnum, ccd=ccd, version=version, ext=ext, prefix=prefix)
                    filename = os.path.basename(uri)
                    storage.copy(filename, uri)

        except Exception as ex:
            message = str(ex)
            logging.error(message)

        storage.set_status(task, prefix, expnums[0], version, ccd, status=message)

    return


def main():
    # Must be running as a script

    parser = argparse.ArgumentParser(
        description='Run step2jmp and step2matt on a given exposure.')

    parser.add_argument("--ccd", "-c",
                        action="store",
                        default=None,
                        type=int,
                        dest="ccd")
    parser.add_argument("--fk", action="store_true", default=False, help="Do fakes?")
    parser.add_argument("--dbimages",
                        action="store",
                        default="vos:OSSOS/dbimages",
                        help='vospace dbimages containerNode')
    parser.add_argument("expnums",
                        type=int,
                        nargs=3,
                        help="3 expnums to process")
    parser.add_argument("--version",
                        action='version',
                        version='%(prog)s 1.0')
    parser.add_argument('-t', '--type',
                        help='which type of image to process',
                        choices=['s', 'p', 'o'],
                        default='p')
    parser.add_argument('--no-sort',
                        help='preserve input exposure order',
                        action='store_true')
    parser.add_argument("--verbose", "-v",
                        action="store_true")
    parser.add_argument("--default", default="WCS",
                        choices=['WCS', 'JMP'],
                        help="Which shift should be used if they dis-agree?")
    parser.add_argument("--debug",
                        action="store_true")
    parser.add_argument("--dry-run", action="store_true", help="run without pushing back to VOSpace, implies --force")
    parser.add_argument("--force", action="store_true")

    cmd_line = " ".join(sys.argv)
    args = parser.parse_args()

    util.set_logger(args)
    logging.info("Starting {}".format(cmd_line))
    storage.DBIMAGES = args.dbimages
    prefix = (args.fk and "fk") or ""
    expnums = args.expnums
    version = args.type

    if args.ccd is None:
        expnum = min(expnums)
        if int(expnum) < 1785619:
            # Last exposures with 36 CCD Megaprime
            ccdlist = list(range(0, 36))
        else:
            # First exposrues with 40 CCD Megaprime
            ccdlist = list(range(0, 40))
    else:
        ccdlist = [args.ccd]

    if not args.no_sort:
        args.expnums.sort()

    for ccd in ccdlist:
        run(args.expnums, ccd, version=version, prefix=prefix, dry_run=args.dry_run, default=args.default, force=args.force)


if __name__ == '__main__':
    sys.exit(main())



############################################################################
#                                                                          #
#  Copyright 2013 by its authors                                           #
#  See COPYING, AUTHORS                                                    #
#                                                                          #
#  This file is part of OSSOS Moving Object Pipeline (OSSOS-MOP)           #
#                                                                          #
#   OSSOS-MOP is free software: you can redistribute it and/or modify      #
#   it under the terms of the GNU General Public License as published by   #
#   the Free Software Foundation, either version 3 of the License, or      #
#   (at your option) any later version.                                    #
#                                                                          #
#   OSSOS-MOP is distributed in the hope that it will be useful,           #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
#   GNU General Public License for more details.                           #
#                                                                          #
#   You should have received a copy of the GNU General Public License      #
#   along with OSSOS-MOP.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                          #
############################################################################
"""plant synthetic moving objects into a set of observations.

prior to planting, the headers of the objects may be swapped around."""

import argparse
import errno
import json
import logging
import math
import os
import sys
import tempfile

from astropy import wcs
from astropy.io import fits
from numpy import radians, fabs, log10, rint, cos, sin, transpose
from pyraf import iraf

from ossos import storage
from ossos import util
from ossos.plant import KBOGenerator

task = 'plant'
dependency = 'mkpsf'


def plant_kbos(filename, psf, kbos, shifts, prefix):
    """
    Add KBOs to an image
    :param filename: name of the image to add KBOs to
    :param psf: Point Spread Function in IRAF/DAOPHOT format, used by ADDSTAR
    :param kbos: list of KBOs to add, has format as returned by KBOGenerator
    :param shifts: dictionary to transform coordinates to reference frame.
    :param prefix: an estimate FWHM of the image, used to determine trailing.
    :return: None
    """

    iraf.set(uparm="./")
    iraf.digiphot()
    iraf.apphot()
    iraf.daophot(_doprint=0)
    iraf.images()

    if shifts['nmag'] < 4:
        logging.warning("Mag shift based on fewer than 4 common stars.")
        fd = open("plant.WARNING", 'a')
        fd.write("Mag shift based on fewer than 4 common stars.")
        fd.close()

    if shifts['emag'] > 0.05:
        logging.warning("Mag shift has large uncertainty.")
        fd = open("plant.WARNING", 'a')
        fd.write("Mag shift hsa large uncertainty.")
        fd.close()

    addstar = tempfile.NamedTemporaryFile(suffix=".add", mode='w')

    # transform KBO locations to this frame using the shifts provided.
    w = get_wcs(shifts)

    header = fits.open(filename)[0].header

    # set the rate of motion in units of pixels/hour instead of ''/hour
    scale = header['PIXSCAL1']
    rate = kbos['sky_rate']/scale

    # compute the location of the KBOs in the current frame.

    # offset magnitudes from the reference frame to the current one.
    mag = kbos['mag'] - shifts['dmag']
    angle = radians(kbos['angle'])

    # Move the x/y locations to account for the sky motion of the source.
    x = kbos['x'] - rate*24.0*shifts['dmjd']*cos(angle)
    y = kbos['y'] - rate*24.0*shifts['dmjd']*sin(angle)
    x, y = w.wcs_world2pix(x, y, 1)

    # Each source will be added as a series of PSFs so that a new PSF is
    # added for each pixel the source moves.
    itime = float(header['EXPTIME'])/3600.0
    npsf = fabs(rint(rate * itime)) + 1
    mag += 2.5*log10(npsf)
    dt_per_psf = itime/npsf

    # Build an addstar file to be used in the planting of source.
    idx = 0
    for record in transpose([x, y, mag, npsf, dt_per_psf, rate, angle]):
        x = record[0]
        y = record[1]
        mag = record[2]
        npsf = record[3]
        dt = record[4]
        rate = record[5]
        angle = record[6]
        for i in range(int(npsf)):
            idx += 1
            x += dt*rate*math.cos(angle)
            y += dt*rate*math.sin(angle)
            addstar.write("{} {} {} {}\n".format(x, y, mag, idx))

    addstar.flush()
    fk_image = prefix+filename
    try:
        os.unlink(fk_image)
    except OSError as err:
        if err.errno == errno.ENOENT:
            pass
        else:
            raise

    # add the sources to the image.
    if os.access(f'{fk_image}.art', os.R_OK):
        os.unlink(f'{fk_image}.art')
    iraf.daophot.addstar(filename, addstar.name, psf, fk_image,
                         simple=True, verify=False, verbose=False)
    # convert the image to short integers.
    iraf.images.chpix(fk_image, fk_image, 'ushort')


def get_wcs(shifts):
    """
    Using the 'shifts' dictionary return a WCS object.

    @param shifts: a dictionary that looks like a fits.header

    Pass in the header WCS parameters (CRVAL, CRPIX, CD) and get a
    astropy WCS object.

    @return wcs.WCS
    """
    # store the shifts as a WCS transform where the 'world' coordinates
    # are really the pixel coordinates
    # in the reference frame.
    return wcs.WCS(header=shifts, naxis=2)


def plant(expnums, ccd, rmin, rmax, ang, width, number=10,
          mmin=21.0, mmax=25.5, version='s', dry_run=False, force=True):
    """Plant artificial sources into the list of images provided.

    @param dry_run: don't push results to VOSpace.
    @param width: The +/- range of angles of motion
    @param ang: The mean angle of motion to add sources
    @param rmax: The maximum rate of motion to add sources at (''/hour)
    @param rmin: The minimum rate of motion to add sources at (''/hour)
    @param expnums: list of MegaPrime exposure numbers to add artificial KBOs to
    @param ccd: which ccd to work on.
    @param mmax: Maximum magnitude to plant sources at
    @param version: Add sources to the 'o', 'p' or 's' images
    @param mmin: Minimum magnitude to plant sources at
    @param number: number of sources to plant.
    @param force: Run, even if we already succeeded at making a fk image.
    """
    message = storage.SUCCESS

    if storage.get_status(task, "", expnums[0], version, ccd) and not force:
        logging.info("{} completed successfully for {}{}{}{:02d}".format(
            task, "", expnums[0], version, ccd))
        return

    with storage.LoggingManager(task, "", expnums[0], ccd, version, dry_run):
        try:
            # Construct a list of artificial KBOs with positions in the image
            # and rates of motion within the bounds given by the caller.
            filename = storage.get_image(expnums[0],
                                         ccd=ccd,
                                         version=version)
            header = fits.open(filename)[0].header
            bounds = util.get_pixel_bounds_from_datasec_keyword(
                header.get('DATASEC', '[33:2080,1:4612]'))

            # generate a set of artificial KBOs to add to the image.
            kbos = KBOGenerator.get_kbos(n=number,
                                         rate=(rmin, rmax),
                                         angle=(ang - width, ang + width),
                                         mag=(mmin, mmax),
                                         x=(bounds[0][0], bounds[0][1]),
                                         y=(bounds[1][0], bounds[1][1]),
                                         filename='Object.planted')

            for expnum in expnums:
                filename = storage.get_image(expnum, ccd, version)
                psf = storage.get_file(expnum, ccd, version, ext='psf.fits')
                plant_kbos(filename, psf, kbos,
                           get_shifts(expnum, ccd, version), "fk")

            if dry_run:
                return
            uri = storage.get_uri('Object', ext='planted', version='',
                                  subdir=f"{expnums[0]}/ccd{int(ccd):02d}")

            storage.copy('Object.planted', uri)
            for expnum in expnums:
                uri = storage.get_uri(expnum,
                                      ccd=ccd,
                                      version=version,
                                      ext='fits', prefix='fk')
                filename = os.path.basename(uri)
                storage.copy(filename, uri)
        except Exception as ex:
            message = str(ex)
            logging.error(message)

        storage.set_status(task, "", expnums[0], version, ccd, status=message)

    return


def get_shifts(expnum, ccd, version):
    return json.loads(open(storage.get_file(expnum, ccd,
                                            version, ext='shifts')).read())


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--ccd',
                        action='store',
                        type=int,
                        default=None,
                        help='which ccd to process')
    parser.add_argument("--dbimages",
                        action="store",
                        default="vos:OSSOS/dbimages",
                        help='vospace dbimages containerNode')
    parser.add_argument("expnums",
                        type=int,
                        nargs=3,
                        help="expnum(s) to process")
    parser.add_argument("--type",
                        action='store',
                        default='s',
                        choices=['s', 'p', 'o'],
                        help='which type of image')
    parser.add_argument('--no-sort',
                        action='store_true',
                        default=False,
                        help='no sorting exposure before processing')
    parser.add_argument("--verbose", "-v",
                        action="store_true")
    parser.add_argument("--debug", '-d',
                        action='store_true')
    parser.add_argument("--number", "-n", type=int,
                        help="Number of KBOs to plant into images.",
                        default=10)
    parser.add_argument("--mmin", type=float,
                        help="Minimum magnitude value to add source with.",
                        default=21.0)
    parser.add_argument("--mmax", type=float,
                        help="Maximum magnitude value to add source with.",
                        default=25.5)
    parser.add_argument("--rmin", default=0.5,
                        type=float, help="minimum motion rate")
    parser.add_argument("--rmax", default=15,
                        type=float, help="maximum motion rate")
    parser.add_argument("--width", default=30,
                        type=float, help="angle opening")
    parser.add_argument("--ang", default=20,
                        type=float, help="angle of motion, 0 is West")
    parser.add_argument("--force", action="store_true")
    parser.add_argument("--dry-run", action="store_true")

    cmd_line = " ".join(sys.argv)
    args = parser.parse_args()

    util.set_logger(args)
    logging.info("Starting {}".format(cmd_line))
    storage.DBIMAGES = args.dbimages
    expnums = args.expnums
    version = args.type

    if args.ccd is None:
        expnum = min(expnums)
        if int(expnum) < 1785619:
            # Last exposures with 36 CCD Megaprime
            ccdlist = list(range(0, 36))
        else:
            # First exposrues with 40 CCD Megaprime
            ccdlist = list(range(0, 40))
    else:
        ccdlist = [args.ccd]

    if not args.no_sort:
        args.expnums.sort()

    for ccd in ccdlist:
        plant(args.expnums,
              ccd,
              args.rmin, args.rmax, args.ang, args.width,
              number=args.number, mmin=args.mmin, mmax=args.mmax,
              version=version,
              dry_run=args.dry_run)


if __name__ == '__main__':
    main()



#!python 
# ###############################################################################
##                                                                            ##
## Copyright 2013 by its authors                                              ##
## See COPYING, AUTHORS                                                       ##
##                                                                            ##
## This file is part of OSSOS Moving Object Pipeline (OSSOS-MOP)              ##
##                                                                            ##
##    OSSOS-MOP is free software: you can redistribute it and/or modify       ##
##    it under the terms of the GNU General Public License as published by    ##
##    the Free Software Foundation, either version 3 of the License, or       ##
##    (at your option) any later version.                                     ##
##                                                                            ##
##    OSSOS-MOP is distributed in the hope that it will be useful,            ##
##    but WITHOUT ANY WARRANTY; without even the implied warranty of          ##
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           ##
##    GNU General Public License for more details.                            ##
##                                                                            ##
##    You should have received a copy of the GNU General Public License       ##
##    along with OSSOS-MOP.  If not, see <http://www.gnu.org/licenses/>.      ##
##                                                                            ##
################################################################################
"""Run the OSSOS make mopheader proceedure"""

import argparse
import logging
import os
from subprocess import CalledProcessError
import sys

from .. import storage
from .. import util
from .. import mopheader

task = "mk_mopheader"
dependency = 'update_header'


def run(expnum, ccd, version, dry_run=False, prefix="", force=False, ignore_dependency=False):
    """Run the OSSOS mopheader script.

    """
    message = storage.SUCCESS
    logging.info("Attempting to get status on header for {} {}".format(expnum, ccd))
    if storage.get_status(task, prefix, expnum, version, ccd) and not force:
        logging.info("{} completed successfully for {} {} {} {}".format(task, prefix, expnum, version, ccd))
        return message

    with storage.LoggingManager(task, prefix, expnum, ccd, version, dry_run):
        try:
            logging.info("Building a mopheader ")
            if not storage.get_status(dependency, prefix, expnum, "p", 36) and not ignore_dependency:
                raise IOError("{} not yet run for {}".format(dependency, expnum))

            # confirm destination directory exists.
            destdir = os.path.dirname(storage.dbimages_uri(expnum, ccd, prefix=prefix, version=version, ext='fits'))
            if not dry_run:
                storage.mkdir(destdir)

            # get image from the vospace storage area
            logging.info("Retrieving image from VOSpace")
            filename = storage.get_image(expnum, ccd, version=version, prefix=prefix)

            # launch the stepZjmp program         
            logging.info("Launching stepZ on %s %d" % (expnum, ccd))
            expname = os.path.basename(filename).split('.')[0]
            logging.info(util.exec_prog(['stepZjmp',
                                         '-f',
                                         expname]))
            # if this is a dry run then we are finished
            if dry_run:
                return message

            # push the header to the VOSpace
            mopheader_filename = expname+".mopheader"
            destination = storage.dbimages_uri(expnum, ccd, prefix=prefix, version=version, ext='mopheader')
            source = mopheader_filename
            count = 0
            with open(source, 'r'):
              while True:
                try:
                    count += 1
                    logging.info("Attempt {} to copy {} -> {}".format(count, source, destination))
                    storage.copy(source, destination)
                    break
                except Exception as ex:
                    if count > 10:
                        raise ex
            logging.info(message)
        except CalledProcessError as cpe:
            message = str(cpe.output)
            logging.error(message)
        except Exception as e:
            message = str(e)
            logging.error(message)

        if not dry_run:
            storage.set_status(task, prefix, expnum, version=version, ccd=ccd, status=message)
        
    return message


def main():
    parser = argparse.ArgumentParser(
        description='Run mopheader chunk of the OSSOS pipeline')

    parser.add_argument('--ccd', '-c',
                        action='store',
                        type=int,
                        dest='ccd',
                        default=None,
                        help='which ccd to process, default is all')
    parser.add_argument('--ignore-update-headers', action='store_true', dest='ignore_update_headers')
    parser.add_argument("--dbimages",
                        action="store",
                        default="vos:OSSOS/dbimages",
                        help='vospace dbimages containerNode')
    parser.add_argument("expnum",
                        type=int,
                        nargs='+',
                        help="expnum(s) to process")
    parser.add_argument("--dry-run",
                        action="store_true",
                        help="DRY RUN, don't copy results to VOSpace, implies --force")

    parser.add_argument("--fk", action="store_true", help="Run fk images")

    parser.add_argument("--type", "-t", choices=['o', 'p', 's'],
                        help="which type of image: o-RAW, p-ELIXIR, s-SCRAMBLE", default='p')
    parser.add_argument("--verbose", "-v",
                        action="store_true")
    parser.add_argument("--force", default=False,
                        action="store_true")
    parser.add_argument("--debug", "-d",
                        action="store_true")

    args = parser.parse_args()

    util.set_logger(args)
    
    logging.info("started")
    prefix = (args.fk and 'fk') or ''

    storage.DBIMAGES = args.dbimages

    if args.ccd is None:
        ccdlist = list(range(0, 36))
    else:
        ccdlist = [args.ccd]

    exit_code = 0
    for expnum in args.expnum:
        for ccd in ccdlist:
            run(expnum, ccd, args.type, args.dry_run, prefix=prefix, force=args.force, ignore_dependency=args.ignore_update_headers)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# coding: utf-8

from astroquery import cadc
import pyds9
import requests
from astropy.io import fits
from io import BytesIO
import numpy as np

client = cadc.Cadc()
query = client.query("""select publisherID, target_name, time_bounds_lower from caom2.Observation o JOIN caom2.Plane p on o.obsID=p.obsID where proposal_id = '15648' AND calibrationLevel = 2""")

target_names = np.unique(query['target_name'])

ds9 = pyds9.DS9('SSOLS')

for target_name in target_names:
    ids = query[query['target_name']==target_name]
    ids = ids[np.argsort(ids['time_bounds_lower'])]
    urls = client.get_data_urls(ids)
    print("Downloading and display {} images for target {}".format(len(urls), target_name))
    ds9.set('frame delete all')
    for url in urls:
        response = requests.get(url)
        stream = BytesIO(response.content)
        hdulist = fits.open(stream)
        ds9.set('frame new')
        ds9.set_fits(hdulist)
    # something to pause downloading.  in DS9 just press a key to move forward.
    junk=ds9.get('iexam key data')







__author__ = "David Rusk <drusk@uvic.ca>"

# NOTE: we are using version 1 of the pubsub API
# (http://pubsub.sourceforge.net/apidocs/docs4v1.html#label-api-v1-docs)
# This is in order to maintain compatiblity with software available on older
# systems such as ScientificLinux 5.5 on CANFAR VMs.

try:
    from pubsub import pub
#    from wx.lib.pubsub import Publisher as pub
except ImportError as ierr:
    print(f"{ierr}")
    print(f"This package now requires PyPubSub to be installed, Hint: pip install pypubsub")
    exit()

# Event ids
ROOT = ("rootid", )
CHANGE_IMAGE = ROOT + ("change_image", )
IMG_LOADED = ROOT + ("imgload", )
FINISHED_WORKUNIT = ROOT + ("finished_workunit", )
NO_AVAILABLE_WORK = ROOT + ("no_available_work", )


def send(event_id, data=None):
    pub.sendMessage(event_id, event=data)


def subscribe(event_id, callback):
    pub.subscribe(callback, event_id)


def unsub_all():
    pub.unsubAll()



"""Compute transformation needed to align images to common x/y grid."""
#
#  Copyright 2013 by its authors
#  See COPYING, AUTHORS
#
#  This file is part of OSSOS Moving Object Pipeline (OSSOS-MOP)
#
#    OSSOS-MOP is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    OSSOS-MOP is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with OSSOS-MOP.  If not, see <http://www.gnu.org/licenses/>.
#
import argparse
import json
import logging
import math
import os
import sys

import numpy
from astropy import wcs
from astropy.io import ascii
from astropy.io import fits
from ossos import daophot
from ossos import storage
from ossos import util

task = 'align'
dependency = 'step2'


class NpEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, numpy.integer):
            return int(obj)
        elif isinstance(obj, numpy.floating):
            return float(obj)
        elif isinstance(obj, numpy.ndarray):
            return obj.tolist()
        else:
            return super(NpEncoder, self).default(obj)


def get_wcs(shifts):
    # store the shifts as a WCS transform where the 'world' coordinates are really the pixel coordinates
    # in the reference frame.
    return wcs.WCS(header=shifts, naxis=2)


def align(expnums, ccd, version='s', prefix='', dry_run=False, force=True):
    """Create a 'shifts' file that transforms the space/flux/time scale of all
    images to the first image.

    This function relies on the .fwhm, .trans.jmp, .phot and .zeropoint.used files for inputs.
    The scaling we are computing here is for use in planting sources into the image at the same sky/flux locations
    while accounting for motions of sources with time.

    :param expnums: list of MegaPrime exposure numbers to add artificial KBOs to,
                    the first frame in the list is the reference.
    :param ccd: which ccd to work on.
    :param prefix: put this string in front of expnum when looking for exposure, normally '' or 'fk'
    :param force: When true run task even if this task is recorded as having succeeded
    :param version: Add sources to the 'o', 'p' or 's' images
    :param dry_run: don't push results to VOSpace.
    """
    message = storage.SUCCESS

    if storage.get_status(task, prefix, expnums[0], version, ccd) and not force:
        logging.info("{} completed successfully for {} {} {} {}".format(task,
                                                                        prefix,
                                                                        expnums[0],
                                                                        version,
                                                                        ccd))
        return

    # Get the images and supporting files that we need from the VOSpace area
    # get_image and get_file check if the image/file is already on disk.
    # re-computed fluxes from the PSF stars and then recompute x/y/flux scaling.

    # some dictionaries to hold the various scale
    pos = {}
    apcor = {}
    mags = {}
    zmag = {}
    mjdates = {}

    with storage.LoggingManager(task, prefix, expnums[0], ccd, version, dry_run):
        try:
            for expnum in expnums:
                filename = storage.get_image(expnum, ccd=ccd, version=version)
                zmag[expnum] = storage.get_zeropoint(expnum, ccd, prefix=None, version=version)
                mjdates[expnum] = float(fits.open(filename)[0].header.get('MJD-OBS'))
                apcor[expnum] = [float(x) for x in open(storage.get_file(expnum,
                                                                         ccd=ccd,
                                                                         version=version,
                                                                         ext=storage.APCOR_EXT)).read().split()]
                keys = ['crval1', 'cd1_1', 'cd1_2', 'crval2', 'cd2_1', 'cd2_2']
                # load the .trans.jmp values into a 'wcs' like dictionary.
                # .trans.jmp maps current frame to reference frame in pixel coordinates.
                # the reference frame of all the frames supplied must be the same.
                shifts = dict(list(zip(keys, [float(x) for x in
                                              open(storage.get_file(expnum,
                                                                    ccd=ccd,
                                                                    version=version,
                                                                    ext='trans.jmp')).read().split()])))
                shifts['crpix1'] = 0.0
                shifts['crpix2'] = 0.0
                # now create a wcs object based on those transforms, this wcs links the current frame's
                # pixel coordinates to the reference frame's pixel coordinates.
                w = get_wcs(shifts)

                # get the PHOT file that was produced by the mkpsf routine
                logging.debug("Reading .phot file {}".format(expnum))
                phot = ascii.read(storage.get_file(expnum, ccd=ccd, version=version, ext='phot'), format='daophot')

                # compute the small-aperture magnitudes of the stars used in the PSF
                logging.debug("Running phot on {}".format(filename))
                mags[expnum] = daophot.phot(filename,
                                            phot['XCENTER'],
                                            phot['YCENTER'],
                                            aperture=apcor[expnum][0],
                                            sky=apcor[expnum][1] + 1,
                                            swidth=apcor[expnum][0],
                                            zmag=zmag[expnum])

                # covert the x/y positions to positions in Frame 1 based on the trans.jmp values.
                logging.debug("Doing the XY translation to refrence frame: {}".format(w))
                (x, y) = w.wcs_pix2world(mags[expnum]["XCENTER"], mags[expnum]["YCENTER"], 1)
                pos[expnum] = numpy.transpose([x, y])
                # match this exposures PSF stars position against those in the first image of the set.
                logging.debug("Matching lists")
                idx1, idx2 = util.match_lists(pos[expnums[0]], pos[expnum])

                # compute the magnitdue offset between the current frame and the reference.
                dmags = numpy.ma.array(mags[expnums[0]]["MAG"] - apcor[expnums[0]][2] -
                                       (mags[expnum]["MAG"][idx1] - apcor[expnum][2]),
                                       mask=idx1.mask)
                dmags.sort()
                # logging.debug("Computed dmags between input and reference: {}".format(dmags))
                error_count = 0

                error_count += 1
                logging.debug("{}".format(error_count))

                # compute the median and determine if that shift is small compared to the scatter.
                try:
                    midx = int(numpy.sum(numpy.any([~dmags.mask], axis=0)) / 2.0)
                    dmag = float(dmags[midx])
                    logging.debug("Computed a mag delta of: {}".format(dmag))
                except Exception as e:
                    logging.error(str(e))
                    logging.error("Failed to compute mag offset between plant and found using: {}".format(dmags))
                    dmag = 99.99

                error_count += 1
                logging.debug("{}".format(error_count))

                try:
                    if math.fabs(dmag) > 3 * (dmags.std() + 0.01):
                        logging.warning("Magnitude shift {} between {} and {} is large: {}".format(dmag,
                                                                                                   expnums[0],
                                                                                                   expnum,
                                                                                                   shifts))
                except Exception as e:
                    logging.error(str(e))

                error_count += 1
                logging.debug("{}".format(error_count))

                shifts['dmag'] = dmag
                shifts['emag'] = dmags.std()
                shifts['nmag'] = len(dmags.mask) - dmags.mask.sum()
                shifts['dmjd'] = mjdates[expnums[0]] - mjdates[expnum]
                shift_file = os.path.basename(storage.get_uri(expnum, ccd, version, '.shifts'))

                error_count += 1
                logging.debug("{}".format(error_count))

                try:
                    fh = open(shift_file, 'w')
                    fh.write(json.dumps(shifts, sort_keys=True, indent=4, separators=(',', ': '),
                                        cls=NpEncoder))
                    fh.write('\n')
                    fh.close()
                except Exception as e:
                    logging.error("Creation of SHIFTS file failed while trying to write: {}".format(shifts))
                    raise e

                error_count += 1
                logging.debug("{}".format(error_count))

                if not dry_run:
                    storage.copy(shift_file, storage.get_uri(expnum, ccd, version, '.shifts'))
            logging.info(message)
        except Exception as ex:
            message = str(ex)
            logging.error(message)

        if not dry_run:
            storage.set_status(task, prefix, expnum, version, ccd, status=message)


def main():
    """
    Run the align process.
    @return: None
    """

    parser = argparse.ArgumentParser()
    parser.add_argument('--ccd',
                        action='store',
                        type=int,
                        default=None,
                        help='which ccd to process')
    parser.add_argument("--dbimages",
                        action="store",
                        default="vos:OSSOS/dbimages",
                        help='vospace dbimages containerNode')
    parser.add_argument("expnums",
                        type=int,
                        nargs=3,
                        help="expnum(s) to process")
    parser.add_argument("--type",
                        action='store',
                        default='s',
                        choices=['s', 'p', 'o'],
                        help='which type of image')
    parser.add_argument("--fk", help="add the fk prefix on processing?",
                        default=False,
                        action='store_true')
    parser.add_argument('--no-sort',
                        action='store_true',
                        default=False,
                        help='do not sort exposure list by expnum before processing')
    parser.add_argument("--verbose", "-v",
                        action="store_true")
    parser.add_argument("--debug", '-d',
                        action='store_true')
    parser.add_argument("--force", action="store_true")
    parser.add_argument("--dry-run", action="store_true")

    cmd_line = " ".join(sys.argv)
    args = parser.parse_args()
    util.set_logger(args)
    logging.info("Starting {}".format(cmd_line))

    storage.DBIMAGES = args.dbimages

    prefix = (args.fk and 'fk') or ''
    version = args.type

    if not args.no_sort:
        args.expnums.sort()

    if args.ccd is None:
        if int(args.expnums[0]) < 1785619:
            # Last exposures with 36 CCD Megaprime
            ccdlist = list(range(0, 36))
        else:
            # First exposrues with 40 CCD Megaprime
            ccdlist = list(range(0, 40))
    else:
        ccdlist = [args.ccd]

    for ccd in ccdlist:
        align(args.expnums, ccd, version=version, prefix=prefix,
              dry_run=args.dry_run)


if __name__ == '__main__':
    main()



###############################################################################
#                                                                            ##
# Copyright 2013 by its authors                                              ##
# See COPYING, AUTHORS                                                       ##
#                                                                            ##
# This file is part of OSSOS Moving Object Pipeline (OSSOS-MOP)              ##
#                                                                            ##
#    OSSOS-MOP is free software: you can redistribute it and/or modify       ##
#    it under the terms of the GNU General Public License as published by    ##
#    the Free Software Foundation, either version 3 of the License, or       ##
#    (at your option) any later version.                                     ##
#                                                                            ##
#    OSSOS-MOP is distributed in the hope that it will be useful,            ##
#    but WITHOUT ANY WARRANTY; without even the implied warranty of          ##
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           ##
#    GNU General Public License for more details.                            ##
#                                                                            ##
#    You should have received a copy of the GNU General Public License       ##
#    along with OSSOS-MOP.  If not, see <http://www.gnu.org/licenses/>.      ##
#                                                                            ##
###############################################################################

import os
import sys
from ossos import storage
from ossos import util
from ossos.pipeline import measure3
import argparse
import logging


task = util.task()
dependency = 'step3'


def run(expnum, ccd, prefix=None, version='p', field=None,
        measure3_dir=storage.MEASURE3, dry_run=False, force=False):

    message = storage.SUCCESS

    if storage.get_status(task, prefix, expnum, version, ccd) and not force:
        logging.info(
            "{} completed successfully for {} {} {} {}".format(task, prefix,
                                                               expnum, version,
                                                               ccd))
        return

    with storage.LoggingManager(task, prefix, expnum, ccd, version, dry_run):
        try:
            if not storage.get_status(dependency, prefix, expnum, version, ccd):
                raise IOError(35, "Cannot start {} as {} not yet completed "
                                  "for {}{}{}{:02d}".format(task, dependency,
                                                            prefix, expnum,
                                                            version, ccd))
            if field is None:
                field = str(expnum)

            if prefix is not None and len(prefix) > 0:
                field = "%s_%s" % (prefix, field)
            field += "_%s%s" % (str(version), str(ccd))

            logging.info("Doing combine on field {}".format(field))

            for ext in ['moving.matt', 'moving.jmp']:
                storage.get_file(expnum, ccd=ccd, version=version, ext=ext,
                                 prefix=prefix)

            # Get the list of objects planted into the field if prefix='fk'
            if prefix == 'fk':
                storage.get_file('Object',
                                 version='',
                                 ext='planted',
                                 subdir=str(expnum) + "/ccd%s" % (
                                     str(ccd).zfill(2)))
            else:
                prefix = ''

            cmd_args = ['comb-list',
                        prefix + str(expnum) + version + str(ccd).zfill(2)]
            logging.info(str(cmd_args))
            logging.info(util.exec_prog(cmd_args))

            # things to copy back to VOSpace, if this is an 'fk' image
            # then we have missed and found files too.
            ext_list = ['cands.comb']
            if prefix == 'fk':
                ext_list.extend(['jmp.missed', 'matt.missed',
                                 'jmp.found', 'matt.found',
                                 'comb.missed', 'comb.found'])

            for ext in ext_list:
                uri = storage.get_uri(expnum,
                                      ccd=ccd,
                                      prefix=prefix,
                                      version=version,
                                      ext=ext)
                filename = os.path.basename(uri)
                if not os.access(filename, os.R_OK):
                    logging.critical("No %s file" % filename)
                    continue
                vospace_name = "%s.%s" % (field, ext)
                if not dry_run:
                    logging.info("%s -> %s" % (
                        filename, os.path.join(measure3_dir, vospace_name)))
                    storage.copy(filename, os.path.join(measure3_dir, vospace_name))

            base_name = prefix + str(expnum) + version + str(ccd).zfill(2)
            cands_file = base_name + '.cands.comb'

            if not os.access(cands_file, os.R_OK):
                no_cands_file = (prefix +
                                 str(expnum) +
                                 version +
                                 str(ccd).zfill(2) +
                                 '.no_candidates')
                open(no_cands_file, 'w').close()
                if not dry_run:
                    vospace_name = "%s.no_candidates" % field
                    storage.copy(no_cands_file,
                                 os.path.join(measure3_dir, vospace_name))

                return storage.SUCCESS

            measure3.run(base_name, storage.DBIMAGES)

            if not dry_run:
                filename = base_name + ".measure3.cands.astrom"
                vospace_filename = "%s.measure3.cands.astrom" % field
                storage.copy(filename, os.path.join(measure3_dir, vospace_filename))

        except Exception as ex:
            message = str(ex)
            logging.error(message)

        if not dry_run:
            storage.set_status(task, prefix, expnum, version, ccd,
                               status=message)


def main():
    parser = argparse.ArgumentParser(
        description='Run step1jmp and step1matt on a given exposure.')

    parser.add_argument("--ccd", "-c",
                        action="store",
                        default=None,
                        type=int,
                        dest="ccd")
    parser.add_argument("--fk", help="add the fk prefix on processing?",
                        default=False,
                        action='store_true')
    parser.add_argument("--dbimages",
                        action="store",
                        default=storage.DBIMAGES,
                        help='vospace dbimages containerNode')
    parser.add_argument("--field",
                        action="store",
                        default=None,
                        help="Name of the field being combined")
    parser.add_argument("expnum",
                        type=int,
                        help="expnum to process")
    parser.add_argument("--measure3",
                        action="store",
                        help="VOSpace location for measure3 files",
                        default=storage.MEASURE3)
    parser.add_argument('--type', default='p', choices=['o', 'p', 's'],
                        help="which type of image")
    parser.add_argument("--verbose", "-v",
                        action="store_true")
    parser.add_argument("--debug", '-d',
                        action='store_true')
    parser.add_argument("--force", '-f', action='store_true')
    parser.add_argument("--dry-run", action="store_true",
                        help="Don't push back to VOSpace, implies --force")

    cmd_line = " ".join(sys.argv)
    args = parser.parse_args()

    util.set_logger(args)
    logging.info("Starting {}".format(cmd_line))

    storage.DBIMAGES = args.dbimages
    storage.MEASURE3 = args.measure3
    expnum = args.expnum
    version = args.type
    prefix = (args.fk and 'fk') or ''

    if args.ccd is None:
        if int(expnum) < 1785619:
            # Last exposures with 36 CCD Megaprime
            ccd_list = list(range(0, 36))
        else:
            # First exposures with 40 CCD Megaprime
            ccd_list = list(range(0, 40))
    else:
        ccd_list = [args.ccd]

    for ccd in ccd_list:
        run(args.expnum,
            ccd,
            prefix=prefix,
            version=version,
            field=args.field,
            measure3_dir=args.measure3,
            dry_run=args.dry_run,
            force=args.force)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# use the xy2skypv code to generate the astrometric values
# that measure3 would normally produce.
import os
import argparse
import logging
from pathlib import Path
from ossos import storage


SUCCESS_FILE = "measure3.OK"
FAILED_EXT = "measure3.FAILED"
CANDS_COMB_EXT = 'cands.comb'
CANDS_ASTROM_EXT = 'measure3.cands.astrom'


def main():
    parser = argparse.ArgumentParser(
        description="Run xy2skypv on cands.comb files to produce cands.astrom")
    parser.add_argument('base_image',
                        help="The base image referencing the .cands.comb file")
    parser.add_argument('--dbimages',
                        help="DBImage VOSpace URI",
                        default=storage.DBIMAGES)

    args = parser.parse_args()
    base_image = args.base_image
    storage.DBIMAGES = args.dbimages
    run(base_image, dbimages=storage.DBIMAGES)


def run(base_image, dbimages=None):
    """
    execute sky2xypv on sources found in cands.comb file.
    """

    if dbimages is not None:
        storage.DBIMAGES = dbimages

    Path(f'{base_image}.{FAILED_EXT}').touch()

    cands_filename = "%s.%s" % (base_image, CANDS_COMB_EXT)
    if not os.access(cands_filename, os.R_OK):
        FileNotFoundError("Failed to open input candidate file %s\n" % cands_filename)

    astrom_header = """##   X        Y        X_0     Y_0          R.A.          DEC                   \n"""

    astrom_filename = "%s.%s" % (base_image, CANDS_ASTROM_EXT)
    coords = []
    base_names = []
    line_counter = 0
    xy_files = {}
    xy_lines = {}
    get_image_names = True
    started = False
    with open(astrom_filename, 'w') as astrom_file:
        cands_lines = open(cands_filename).readlines()
        for cands_line in cands_lines:
            # read the names of the exposures to work from...
            if len(cands_line.strip()) == 0:
                # Skip EMPTY lines...
                continue
            if cands_line.lstrip()[0] == '#':
                # write out all the header lines except the one with column
                # names as its different for the astrom version 'X_0' is a
                # column name in the cands.comb files.
                if "X_0" not in cands_line:
                    astrom_file.write(cands_line)
                else:
                    astrom_file.write(astrom_header)
                if cands_line.lstrip()[1] == '#':
                    get_image_names = not started
                    continue
                logging.info("Read this line: {}".format(cands_line[2:].strip()))
                if get_image_names and cands_line.lstrip()[1] == ' ':
                    base_name = cands_line[2:].strip()
                    base_names.append(base_name)
                    xy_files[base_name] = open("%s.xy" % base_name, 'w')
                    started = True
                    continue
                continue
            v = cands_line.strip().split()
            coords.append(v)
            base_name = base_names[line_counter % len(base_names)]
            if base_name not in xy_lines:
                xy_lines[base_name] = []
            xy_lines[base_name].append(cands_line)
            xy_files[base_name].write("%s %s\n" % (v[0], v[1]))
            xy_files[base_name].flush()
            line_counter += 1

        # Now run the astrometry for each object on the frame that
        # object was detected on

        astrometry_lines = {}
        for base_name in xy_files:
            storage.get_frame(base_name)
            xy_files[base_name].close()
            cmd = 'xy2skypv %s.fits %s.xy %s.rd' % (base_name,
                                                    base_name, base_name)
            os.system(cmd)
            astrometry_lines[base_name] = open("%s.rd" % base_name).readlines()

        for idx in range(len(astrometry_lines[base_names[0]])):
            astrom_file.write("\n")
            for base_name in base_names:
                rd = astrometry_lines[base_name][idx].split()
                xy = xy_lines[base_name][idx].split()
                if rd[2] != xy[0] or rd[3] != xy[1]:
                    # the files are misalligned?
                    raise ValueError("MISMATCH in astrometric code... bailing out.\n")
                astrom_file.write(" %8.2f %8.2f %8.2f %8.2f %12.7f %12.7f\n" % (float(xy[0]),
                                                                                float(xy[1]),
                                                                                float(xy[2]),
                                                                                float(xy[3]),
                                                                                float(rd[0]),
                                                                                float(rd[1])))

    Path(f'{SUCCESS_FILE}').touch()
    os.unlink(f'{base_image}.{FAILED_EXT}')


if __name__ == '__main__':
    main()



import re
from setuptools import setup, find_packages

VERSION_FILENAME = "ossos/version.py"
line = open(VERSION_FILENAME, "rt").read()
VERSION_RE = r"^__version__ = ['\"]([^'\"]*)['\"]"
mo = re.search(VERSION_RE, line, re.M)
if mo:
    version_string = mo.group(1)
else:
    raise RuntimeError("Unable to find version string in %s." % (VERSION_FILENAME,))

dependencies = ['requests >= 2.7',
                'astropy >= 4.0',
                'vos >= 3.3.2',
                'numpy >= 1.6.1',
                'matplotlib',
                'd2to1 >= 0.2.10',
                'scipy',
                'uncertainties',
                'pyds9 >= 1.8',
                'wxPython > 4.0',
                'pypubsub > 4.0',
                'mp_ephem']

console_scripts = ['mkpsf = ossos.pipeline.mkpsf:main', 'step3 = ossos.pipeline.step3:main',
                   'step2 = ossos.pipeline.step2:main', 'step1 = ossos.pipeline.step1:main',
                   'combine = ossos.pipeline.combine:main',
                   'mk_mopheader = ossos.pipeline.mk_mopheader:main',
                   'optimize_pointings = ossos.planning.optimize_pointings:main',
                   'build_astrometry_report = ossos.pipeline.build_astrometry_report:main',
                   'update_astrometry = ossos.pipeline.update_astrometry:main',
                   'measure3 = ossos.pipeline.measure3:main',
                   'align = ossos.pipeline.align:main',
                   'plant = ossos.pipeline.plant:main',
                   'astrom_mag_check = ossos.pipeline.astrom_mag_check:main',
                   'scramble = ossos.pipeline.scramble:main']

gui_scripts = ['validate = ossos.tools.validate:main']

setup(name='ossos',
      version=version_string,
      url='http://github.com/OSSOS/MOP',
      author='''JJ Kavelaars (jjk@uvic.ca),
              Michele Bannister (micheleb@uvic.ca),
              David Rusk''',
      maintainer='M Bannister and JJ Kavelaars',
      maintainer_email='jjk@uvic.ca',
      description="Outer Solar System Origins Survey (OSSOS)",
      long_description='See http://www.ossos-survey.org/ for science details.',
      classifiers=['Intended Audience :: Science/Research',
                   'Topic :: Scientific/Engineering :: Astronomy',
                   'Development Status :: 4 - Beta',
                   'Programming Language :: Python :: 2 :: Only',
                   'Operating System :: MacOS :: MacOS X',
                   'Environment :: X11 Applications',
                   'License :: OSI Approved :: GNU General Public License (GPL)',
                   ],
      package_data={'ossos': ['gui/*.json']},
      install_requires=dependencies,
      # scripts=["scripts/gui",],
      entry_points={'console_scripts': console_scripts,
                    'gui_scripts': gui_scripts},
      packages=find_packages(exclude=['tests', ])
      )
